@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef QMT_SHAPE_H
-#define QMT_SHAPE_H
+#pragma once
 
 namespace qmt {
 
@@ -36,11 +35,9 @@ class IShape
 public:
     virtual ~IShape() { }
 
-    virtual IShape *Clone() const = 0;
+    virtual IShape *clone() const = 0;
     virtual void accept(ShapeVisitor *visitor) = 0;
     virtual void accept(ShapeConstVisitor *visitor) const = 0;
 };
 
 } // namespace qmt
-
-#endif // QMT_SHAPE_H
