@@ -29,6 +29,7 @@
 
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
+#include <utils/stringutils.h>
 
 #include <QDir>
 #include <QFile>
@@ -46,9 +47,7 @@ MakefileParser::~MakefileParser()
 
 bool MakefileParser::parse()
 {
-    m_mutex.lock();
     m_cancel = false;
-    m_mutex.unlock();
 
     m_success = true;
     m_executable.clear();
@@ -124,13 +123,11 @@ QStringList MakefileParser::cxxflags() const
 
 void MakefileParser::cancel()
 {
-    QMutexLocker locker(&m_mutex);
     m_cancel = true;
 }
 
 bool MakefileParser::isCanceled() const
 {
-    QMutexLocker locker(&m_mutex);
     return m_cancel;
 }
 
@@ -153,7 +150,7 @@ MakefileParser::TopTarget MakefileParser::topTarget() const
         return BuiltSources;
     if (id == QLatin1String("SUBDIRS") || id == QLatin1String("DIST_SUBDIRS"))
         return SubDirs;
-    if (id.endsWith(QLatin1String("_SOURCES")))
+    if (id.endsWith("_SOURCES") || id.endsWith("_HEADERS"))
         return Sources;
 
     return Undefined;
@@ -173,7 +170,7 @@ void MakefileParser::parseBinPrograms()
 
 void MakefileParser::parseSources()
 {
-    QTC_ASSERT(m_line.contains(QLatin1String("_SOURCES")), return);
+    QTC_ASSERT(m_line.contains("_SOURCES") || m_line.contains("_HEADERS"), return);
 
     bool hasVariables = false;
     m_sources.append(targetValues(&hasVariables));
@@ -375,7 +372,7 @@ QStringList MakefileParser::targetValues(bool *hasVariables)
         // Get all values of a line separated by spaces.
         // Values representing a variable like $(value) get
         // removed currently.
-        QStringList lineValues = m_line.split(QLatin1Char(' '), QString::SkipEmptyParts);
+        QStringList lineValues = m_line.split(QLatin1Char(' '), Qt::SkipEmptyParts);
         QStringList::iterator it = lineValues.begin();
         while (it != lineValues.end()) {
             if ((*it).startsWith(QLatin1String("$("))) {
@@ -444,7 +441,7 @@ QStringList MakefileParser::parseTermsAfterAssign(const QString &line)
     if (assignPos <= 0 || assignPos >= line.size())
         return QStringList();
 
-    const QStringList parts = Utils::QtcProcess::splitArgs(line.mid(assignPos));
+    const QStringList parts = Utils::ProcessArgs::splitArgs(line.mid(assignPos));
     QStringList result;
     for (int i = 0; i < parts.count(); ++i) {
         const QString cur = parts.at(i);
