@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,44 +9,41 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "customwizardparameters.h"
-#include "customwizardpreprocessor.h"
 #include "customwizardscriptgenerator.h"
 
-#include <coreplugin/mimedatabase.h>
 #include <coreplugin/icore.h>
-#include <cpptools/cpptoolsconstants.h>
+#include <cppeditor/cppeditorconstants.h>
 
+#include <utils/macroexpander.h>
+#include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
+#include <utils/templateengine.h>
+#include <utils/temporarydirectory.h>
+#include <utils/temporaryfile.h>
 
 #include <QCoreApplication>
 #include <QDate>
 #include <QDebug>
-#include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QIcon>
-#include <QScriptEngine>
-#include <QTemporaryFile>
+#include <QJSEngine>
 #include <QTime>
 #include <QXmlStreamAttribute>
 #include <QXmlStreamReader>
@@ -54,6 +51,7 @@
 enum { debug = 0 };
 
 using namespace Core;
+using namespace Utils;
 
 static const char customWizardElementC[] = "wizard";
 static const char iconElementC[] = "icon";
@@ -177,7 +175,7 @@ bool CustomWizardValidationRule::validateRules(const QList<CustomWizardValidatio
     errorMessage->clear();
     if (rules.isEmpty())
         return true;
-    QScriptEngine engine;
+    QJSEngine engine;
     foreach (const CustomWizardValidationRule &rule, rules)
     if (!rule.validate(engine, replacementMap)) {
         *errorMessage = rule.message;
@@ -187,14 +185,14 @@ bool CustomWizardValidationRule::validateRules(const QList<CustomWizardValidatio
     return true;
 }
 
-bool CustomWizardValidationRule::validate(QScriptEngine &engine, const QMap<QString, QString> &replacementMap) const
+bool CustomWizardValidationRule::validate(QJSEngine &engine, const QMap<QString, QString> &replacementMap) const
 {
     // Apply parameters and evaluate using JavaScript
     QString cond = condition;
     CustomWizardContext::replaceFields(replacementMap, &cond);
     bool valid = false;
     QString errorMessage;
-    if (!evaluateBooleanJavaScriptExpression(engine, cond, &valid, &errorMessage)) {
+    if (!Utils::TemplateEngine::evaluateBooleanJavaScriptExpression(engine, cond, &valid, &errorMessage)) {
         qWarning("Error in custom wizard validation expression '%s': %s",
                  qPrintable(cond), qPrintable(errorMessage));
         return false;
@@ -202,11 +200,6 @@ bool CustomWizardValidationRule::validate(QScriptEngine &engine, const QMap<QStr
     return valid;
 }
 
-CustomWizardParameters::CustomWizardParameters() :
-        firstPageId(-1)
-{
-}
-
 void CustomWizardParameters::clear()
 {
     directory.clear();
@@ -254,7 +247,7 @@ static inline bool assignLanguageElementText(QXmlStreamReader &reader,
                                              const QString &desiredLanguage,
                                              QString *target)
 {
-    const QStringRef elementLanguage = reader.attributes().value(QLatin1String(langAttributeC));
+    const auto elementLanguage = reader.attributes().value(QLatin1String(langAttributeC));
     if (elementLanguage.isEmpty()) {
         // Try to find a translation for our built-in Wizards
         *target = QCoreApplication::translate("ProjectExplorer::CustomWizard", reader.readElementText().toLatin1().constData());
@@ -275,7 +268,7 @@ static bool parseCustomProjectElement(QXmlStreamReader &reader,
                                       const QString &language,
                                       CustomWizardParameters *p)
 {
-    const QStringRef elementName = reader.name();
+    const auto elementName = reader.name();
     if (elementName == QLatin1String(iconElementC)) {
         const QString path = reader.readElementText();
         const QIcon icon = wizardIcon(configFileFullPath, path);
@@ -315,7 +308,7 @@ static inline QMap<QString, QString> attributesToStringMap(const QXmlStreamAttri
 }
 
 // Switch parser state depending on opening element name.
-static ParseState nextOpeningState(ParseState in, const QStringRef &name)
+static ParseState nextOpeningState(ParseState in, QStringView name)
 {
     switch (in) {
     case ParseBeginning:
@@ -382,7 +375,7 @@ static ParseState nextOpeningState(ParseState in, const QStringRef &name)
 }
 
 // Switch parser state depending on closing element name.
-static ParseState nextClosingState(ParseState in, const QStringRef &name)
+static ParseState nextClosingState(ParseState in, QStringView name)
 {
     switch (in) {
     case ParseBeginning:
@@ -450,32 +443,26 @@ static ParseState nextClosingState(ParseState in, const QStringRef &name)
 // Parse kind attribute
 static inline IWizardFactory::WizardKind kindAttribute(const QXmlStreamReader &r)
 {
-    const QStringRef value = r.attributes().value(QLatin1String(kindAttributeC));
-    if (!value.isEmpty()) {
-        if (value == QLatin1String("file"))
-            return IWizardFactory::FileWizard;
-        if (value == QLatin1String("class"))
-            return IWizardFactory::ClassWizard;
-    }
+    const auto value = r.attributes().value(QLatin1String(kindAttributeC));
+    if (value == QLatin1String("file") || value == QLatin1String("class"))
+        return IWizardFactory::FileWizard;
     return IWizardFactory::ProjectWizard;
 }
 
-static inline FeatureSet readRequiredFeatures(const QXmlStreamReader &reader)
+static inline QSet<Id> readRequiredFeatures(const QXmlStreamReader &reader)
 {
     QString value = reader.attributes().value(QLatin1String(featuresRequiredC)).toString();
-    QStringList stringList = value.split(QLatin1Char(','), QString::SkipEmptyParts);
-    FeatureSet features;
-    foreach (const QString &string, stringList) {
-        Feature feature(Id::fromString(string));
-        features |= feature;
-    }
+    QStringList stringList = value.split(QLatin1Char(','), Qt::SkipEmptyParts);
+    QSet<Id> features;
+    foreach (const QString &string, stringList)
+        features |= Id::fromString(string);
     return features;
 }
 
 static inline IWizardFactory::WizardFlags wizardFlags(const QXmlStreamReader &reader)
 {
     IWizardFactory::WizardFlags flags;
-    const QStringRef value = reader.attributes().value(QLatin1String(platformIndependentC));
+    const auto value = reader.attributes().value(QLatin1String(platformIndependentC));
 
     if (!value.isEmpty() && value == QLatin1String("true"))
         flags |= IWizardFactory::PlatformIndependent;
@@ -494,7 +481,7 @@ static inline QString msgError(const QXmlStreamReader &reader,
 static inline bool booleanAttributeValue(const QXmlStreamReader &r, const char *nameC,
                                          bool defaultValue)
 {
-    const QStringRef attributeValue = r.attributes().value(QLatin1String(nameC));
+    const auto attributeValue = r.attributes().value(QLatin1String(nameC));
     if (attributeValue.isEmpty())
         return defaultValue;
     return attributeValue == QLatin1String("true");
@@ -502,7 +489,7 @@ static inline bool booleanAttributeValue(const QXmlStreamReader &r, const char *
 
 static inline int integerAttributeValue(const QXmlStreamReader &r, const char *name, int defaultValue)
 {
-    const QStringRef sValue = r.attributes().value(QLatin1String(name));
+    const auto sValue = r.attributes().value(QLatin1String(name));
     if (sValue.isEmpty())
         return defaultValue;
     bool ok;
@@ -583,7 +570,11 @@ CustomWizardParameters::parse(QIODevice &device, const QString &configFileFullPa
                 case ParseWithinWizard:
                     if (!booleanAttributeValue(reader, wizardEnabledAttributeC, true))
                         return ParseDisabled;
-                    id = attributeValue(reader, idAttributeC);
+                    {
+                        const QString idString = attributeValue(reader, idAttributeC);
+                        if (!idString.isEmpty())
+                            id = Utils::Id::fromString(idString);
+                    }
                     category = attributeValue(reader, categoryAttributeC);
                     kind = kindAttribute(reader);
                     requiredFeatures = readRequiredFeatures(reader);
@@ -846,8 +837,8 @@ bool CustomWizardContext::replaceFields(const FieldReplacementMap &fm, QString *
 // used for the arguments of a generator script.
 class TemporaryFileTransform {
 public:
-    typedef CustomWizardContext::TemporaryFilePtr TemporaryFilePtr;
-    typedef CustomWizardContext::TemporaryFilePtrList TemporaryFilePtrList;
+    using TemporaryFilePtr = CustomWizardContext::TemporaryFilePtr;
+    using TemporaryFilePtrList = CustomWizardContext::TemporaryFilePtrList;
 
     explicit TemporaryFileTransform(TemporaryFilePtrList *f);
 
@@ -859,16 +850,12 @@ private:
 };
 
 TemporaryFileTransform::TemporaryFileTransform(TemporaryFilePtrList *f) :
-    m_files(f), m_pattern(QDir::tempPath())
-{
-    if (!m_pattern.endsWith(QLatin1Char('/')))
-        m_pattern += QLatin1Char('/');
-    m_pattern += QLatin1String("qtcreatorXXXXXX.txt");
-}
+    m_files(f), m_pattern(Utils::TemporaryDirectory::masterDirectoryPath() + "/qtcreatorXXXXXX.txt")
+{ }
 
 QString TemporaryFileTransform::operator()(const QString &value) const
 {
-    TemporaryFilePtr temporaryFile(new QTemporaryFile(m_pattern));
+    TemporaryFilePtr temporaryFile(new Utils::TemporaryFile(m_pattern));
     QTC_ASSERT(temporaryFile->open(), return QString());
 
     temporaryFile->write(value.toLocal8Bit());
@@ -918,9 +905,11 @@ void CustomWizardContext::reset()
     const QTime currentTime = QTime::currentTime();
     baseReplacements.clear();
     baseReplacements.insert(QLatin1String("CppSourceSuffix"),
-                            MimeDatabase::preferredSuffixByType(QLatin1String(CppTools::Constants::CPP_SOURCE_MIMETYPE)));
+                            Utils::mimeTypeForName(QLatin1String(CppEditor::Constants::CPP_SOURCE_MIMETYPE))
+                            .preferredSuffix());
     baseReplacements.insert(QLatin1String("CppHeaderSuffix"),
-                            MimeDatabase::preferredSuffixByType(QLatin1String(CppTools::Constants::CPP_HEADER_MIMETYPE)));
+                            Utils::mimeTypeForName(QLatin1String(CppEditor::Constants::CPP_HEADER_MIMETYPE))
+                            .preferredSuffix());
     baseReplacements.insert(QLatin1String("CurrentDate"),
                             currentDate.toString(Qt::ISODate));
     baseReplacements.insert(QLatin1String("CurrentTime"),
@@ -934,9 +923,9 @@ void CustomWizardContext::reset()
     baseReplacements.insert(QLatin1String("CurrentTime:RFC"),
                             currentTime.toString(Qt::RFC2822Date));
     baseReplacements.insert(QLatin1String("CurrentDate:Locale"),
-                            currentDate.toString(Qt::DefaultLocaleShortDate));
+                            QLocale::system().toString(currentDate, QLocale::ShortFormat));
     baseReplacements.insert(QLatin1String("CurrentTime:Locale"),
-                            currentTime.toString(Qt::DefaultLocaleShortDate));
+                            QLocale::system().toString(currentTime, QLocale::ShortFormat));
     replacements.clear();
     path.clear();
     targetPath.clear();
@@ -952,8 +941,18 @@ QString CustomWizardContext::processFile(const FieldReplacementMap &fm, QString
         replaceFields(fm, &in);
 
     QString out;
+
+    // Expander needed to handle extra variable "Cpp:PragmaOnce"
     QString errorMessage;
-    if (!customWizardPreprocess(in, &out, &errorMessage)) {
+    Utils::MacroExpander *expander = Utils::globalMacroExpander();
+    in = Utils::TemplateEngine::processText(expander, in, &errorMessage);
+    if (!errorMessage.isEmpty()) {
+        qWarning("Error processing custom widget file: %s\nFile:\n%s",
+                 qPrintable(errorMessage), qPrintable(in));
+        return QString();
+    }
+
+    if (!Utils::TemplateEngine::preprocessText(in, &out, &errorMessage)) {
         qWarning("Error preprocessing custom widget file: %s\nFile:\n%s",
                  qPrintable(errorMessage), qPrintable(in));
         return QString();
