@@ -107,11 +107,12 @@ bool Cursor::isDeclaration() const
 
 bool Cursor::isInvalidDeclaration() const
 {
-#ifdef IS_INVALIDDECL_SUPPORTED
     return clang_isInvalidDeclaration(m_cxCursor);
-#else
-    return false;
-#endif
+}
+
+bool Cursor::isParameter() const
+{
+    return kind() == CXCursor_ParmDecl;
 }
 
 bool Cursor::isLocalVariable() const
@@ -250,9 +251,11 @@ ClangString Cursor::briefComment() const
     return ClangString(clang_Cursor_getBriefCommentText(m_cxCursor));
 }
 
-ClangString Cursor::rawComment() const
+Utf8String Cursor::rawComment() const
 {
-    return ClangString(clang_Cursor_getRawCommentText(m_cxCursor));
+    Utf8String comment = ClangString(clang_Cursor_getRawCommentText(m_cxCursor));
+    comment.replace(Utf8String("\r\n"), Utf8String("\n"));
+    return comment;
 }
 
 int Cursor::argumentCount() const
